package main

import (
	"git.kuschku.de/justjanne/stateless-matrix-bot/api"
	"io"
	"log"
	"net/http"
	"strings"
)

type MatrixBot struct {
	userData api.LoginResponse
	handlers map[string]func(bot *MatrixBot, notification api.Notification) error
}

func NewMatrixBot(
	username string,
	password string,
	deviceId string,
	url string,
) (*MatrixBot, error) {
	userData, err := api.Login(username, password, deviceId)
	if err != nil {
		return nil, err
	}
	err = api.SetPusher(userData, url)
	if err != nil {
		return nil, err
	}
	return &MatrixBot{
		userData: userData,
		handlers: make(map[string]func(bot *MatrixBot, notification api.Notification) error),
	}, nil
}

func (bot *MatrixBot) HandleFunc(command string, handler func(bot *MatrixBot, notification api.Notification) error) {
	bot.handlers[command] = handler
}

func (bot *MatrixBot) Serve(endpoint string) {
	http.HandleFunc("/healthz", func(writer http.ResponseWriter, request *http.Request) {
		io.WriteString(writer, "OK\n")
	})
	http.HandleFunc("/_matrix/push/v1/notify", func(writer http.ResponseWriter, request *http.Request) {
		notification, err := api.ParseNotification(request.Body)
		if err != nil {
			log.Println(err.Error())
			return
		}
		if notification.EventId == "" {
			return
		}
		err = api.SetReadReceipt(bot.userData, notification.RoomId, notification.EventId)
		if err != nil {
			log.Println(err.Error())
			return
		}
		command := strings.SplitN(notification.Content.Body, " ", 2)[0]
		handler, ok := bot.handlers[command]
		if !ok {
			log.Printf("could not find handler for '%s'\n", command)
			return
		}
		err = handler(bot, notification)
		if err != nil {
			log.Println(err.Error())
			return
		}
	})
	log.Fatal(http.ListenAndServe(endpoint, nil))
}
